from enum import Enum
from typing import TypedDict


class BankId(str, Enum):
    ABSA = 'absa'
    Capitec = 'capitec'
    Discovery = 'discovery_bank'
    FNB = 'fnb'
    Investec = 'investec'
    Nedbank = 'nedbank'
    StandardBank = 'standard_bank'
    TymeBank = 'tymebank'
    ZAAccessBank = 'za_access_bank'
    AccessBank = 'access_bank'
    FirstBankOfNigeria = 'first_bank_of_nigeria'
    GTBank = 'gtbank'
    ProvidusBank = 'providus_bank'
    SterlingBank = 'sterling_bank'
    UnitedBankForAfrica = 'united_bank_for_africa'
    VFDMicrofinanceBank = 'vfd_microfinance_bank'
    WemaBank = 'wema_bank'
    ZenithBank = 'zenith_bank'


class Currency(TypedDict):
    quantity: str
    currency: str



from enum import Enum
from typing import TypedDict

from gql import gql

from .shared_types import BankId


class Payer(TypedDict):
    email: str
    name: str
    reference: str
    phoneNumber: str


class BeneficiaryType(str, Enum):
    Public = 'public'
    Private = 'private'


class AccountType(str, Enum):
    Current = 'current'
    Savings = 'savings'
    Credit = 'credit'
    Loan = 'loan'
    Investment = 'investment'
    Other = 'other'
    Unknown = 'unknown'


class LinkPayBankAccount(TypedDict):
    name: str
    bankId: BankId
    accountNumber: str
    accountType: AccountType
    beneficiaryType: BeneficiaryType
    reference: str


class InitialPayment(TypedDict):
    amount: str
    external_reference: str


query = gql(
    """
    mutation CreatePaymentAuthorisation(
      $beneficiary: ClientPaymentAuthorizationRequestCreateBeneficiaryInput!,
      $payer: ClientPaymentAuthorizationRequestCreatePayerInput!,
      $initialPayment: ClientPaymentAuthorizationRequestCreateInitialPaymentInput,
    ) {
      clientPaymentAuthorizationRequestCreate(
        input: {
          beneficiary: $beneficiary,
          payer: $payer,
          initialPayment: $initialPayment,
        }
      ) {
        authorizationRequestUrl
      }
    }
    """,
)



from stitch_sdk.client import Stitch
from stitch_sdk.queries.create_payment_authorisation import (
    AccountType,
    BeneficiaryType,
    LinkPayBankAccount,
    Payer,
)
from stitch_sdk.queries.create_payment_request import InstantPayBankAccount
from stitch_sdk.queries.shared_types import BankId


def example_create_payment_authorisation(client: Stitch) -> str:
    bank_account: LinkPayBankAccount = {
        'name': 'Sample Account',
        'bankId': BankId.ABSA,
        'accountNumber': '1234567890',
        'accountType': AccountType.Current,
        'beneficiaryType': BeneficiaryType.Private,
        'reference': 'TestBeneficiary',
    }
    payer: Payer = {
        'email': 'sampleuser@example.com',
        'name': 'Sample User',
        'reference': 'TestPayer',
        'phoneNumber': '27821234567',
    }

    url = client.create_payment_authorisation(bank_account, payer, 'https://localhost:8000/return')

    return url


def example_create_payment_request(client: Stitch) -> str:
    bank_account: InstantPayBankAccount = {
        'name': 'Sample Account',
        'bankId': BankId.ABSA,
        'accountNumber': '1234567890',
    }
    payer: Payer = {
        'email': 'sampleuser@example.com',
        'name': 'Sample User',
        'reference': 'TestPayer',
        'phoneNumber': '27821234567',
    }

    url = client.create_payment_request(
        amount={'currency': 'ZAR', 'quantity': '123.32'},
        payer_reference='Test Ref',
        beneficiary_reference='Ben Ref',
        external_reference='Ext Ref',
        bank_account=bank_account,
        redirect_url='https://localhost:8000/return',
    )

    return url


def main():
    client = Stitch(
        url='https://api.stitch.money/graphql',
        cert_path='./certificate.pem',
        client_id='test-1111111-1111-1111-1111-1111111111',
    )

    payment_auth_url = example_create_payment_authorisation(client)
    payment_request_url = example_create_payment_request(client)

    print(payment_auth_url)
    print(payment_request_url)


if __name__ == '__main__':
    main()



from typing import TypedDict

from gql import gql

from .shared_types import BankId


class InstantPayBankAccount(TypedDict):
    name: str
    bankId: BankId
    accountNumber: str


query = gql(
    """
    mutation CreatePaymentRequest(
      $amount: MoneyInput!,
      $payerReference: String!,
      $beneficiaryReference: String!,
      $externalReference: String,
      $beneficiary: BeneficiaryInput!,
    ) {
      clientPaymentInitiationRequestCreate(
        input: {
          amount: $amount,
          payerReference: $payerReference,
          beneficiaryReference: $beneficiaryReference,
          externalReference: $externalReference,
          beneficiary: $beneficiary,
        }
      ) {
        paymentInitiationRequest {
          id
          url
        }
      }
    }
    """,
)



from setuptools import find_packages, setup


with open('./requirements/base.txt', 'r') as f:
    requirements = f.read().split('\n')


with open('./README.md', 'r') as f:
    readme = f.read()


setup(
    name='stitch_sdk',
    version='v0.1.0',
    description='Unofficial Stitch Python SDK',
    long_description=readme,
    install_requires=requirements,
    url='https://github.com/deanpienaar/stitch-py',
    author='Dean Pienaar',
    license='MIT',
    packages=find_packages(exclude=['tests']),
)



from dataclasses import dataclass, field
import time
from urllib.parse import urlencode
from uuid import uuid4 as uuid

from gql import Client
from gql.transport.exceptions import TransportQueryError
from gql.transport.requests import RequestsHTTPTransport
from graphql import DocumentNode
import jwt
import requests

from .exceptions import StitchAPIError
from .queries import create_payment_authorisation, create_payment_request
from .queries.create_payment_authorisation import InitialPayment, LinkPayBankAccount, Payer
from .queries.create_payment_request import InstantPayBankAccount
from .queries.shared_types import Currency


@dataclass
class Stitch:
    url: str
    cert_path: str
    client_id: str

    _gql_client: Client | None = field(init=False, default=None)

    def create_payment_authorisation(
        self,
        bank_account: LinkPayBankAccount,
        payer: Payer,
        redirect_url: str,
        initial_payment: InitialPayment = None,
    ) -> str:
        result = self._make_query(
            create_payment_authorisation.query,
            variables={
                'beneficiary': {'bankAccount': bank_account},
                'payer': payer,
                'initialPayment': initial_payment,
            },
        )

        url = result['clientPaymentAuthorizationRequestCreate']['authorizationRequestUrl']

        return self._build_redirect_url(url, redirect_url)

    def create_payment_request(
        self,
        amount: Currency,
        payer_reference: str,
        beneficiary_reference: str,
        external_reference: str,
        bank_account: InstantPayBankAccount,
        redirect_url: str,
    ):
        result = self._make_query(
            create_payment_request.query,
            variables={
                'amount': amount,
                'payerReference': payer_reference,
                'beneficiaryReference': beneficiary_reference,
                'externalReference': external_reference,
                'beneficiary': {'bankAccount': bank_account},
            },
        )

        payment_request = result['clientPaymentInitiationRequestCreate']['paymentInitiationRequest']
        url = payment_request['url']

        return self._build_redirect_url(url, redirect_url)

    @property
    def gql_client(self) -> Client:
        if self._gql_client is None:
            client_token = self._get_client_token()
            auth_header = {'Authorization': f'Bearer {client_token}'}

            transport = RequestsHTTPTransport(self.url, headers=auth_header)
            client = Client(transport=transport)

            self._gql_client = client

        return self._gql_client

    def reset_auth(self):
        self._gql_client = None

    def _make_query(self, document: DocumentNode, variables: dict[str] = None) -> dict:
        try:
            return self.gql_client.execute(document, variable_values=variables)
        except TransportQueryError as error:
            stitch_error = Stitch._extract_stitch_error(error.errors)

            if stitch_error is None:
                raise error

            raise StitchAPIError(stitch_error['extensions']['code'], stitch_error['message'])

    @staticmethod
    def _extract_stitch_error(errors: list[dict[str, str]]) -> dict | None:
        for error in errors:
            if error.get('extensions', {}).get('code'):
                return error

    def _get_client_token(self) -> str:
        url = 'https://secure.stitch.money/connect/token'
        client_assertion = self._generate_client_assertion()

        response = requests.post(
            url,
            data={
                'client_id': self.client_id,
                'scope': 'client_paymentauthorizationrequest client_paymentrequest',
                'grant_type': 'client_credentials',
                'audience': url,
                'client_assertion_type': 'urn:ietf:params:oauth:client-assertion-type:jwt-bearer',
                'client_assertion': client_assertion,
            },
        ).json()

        return response['access_token']

    def _generate_client_assertion(self) -> str:
        with open(self.cert_path, 'r') as cert:
            secret = cert.read()

            now = int(time.time())
            one_hour_from_now = now + 3600

            payload = {
                'aud': 'https://secure.stitch.money/connect/token',
                'iss': self.client_id,
                'sub': self.client_id,
                'jti': str(uuid()),
                'iat': now,
                'nbf': now,
                'exp': one_hour_from_now,
            }

            return jwt.encode(payload, secret, algorithm='RS256')

    @staticmethod
    def _build_redirect_url(base_url: str, redirect_url: str) -> str:
        return f'{base_url}?{urlencode({"redirect_uri": redirect_url})}'
